@@ -30,6 +30,13 @@
 #include "seahorse-gconf.h"
 #include "seahorse-gtkstock.h"
 
+/**
+ * SECTION:seahorse-widget
+ * @short_description: wrapping gtk-builder widgets to simplify usage.
+ * @include:seahorse-widget.h
+ *
+ **/
+
 #define STATUS "status"
 
 enum {
@@ -37,9 +44,17 @@ enum {
 	PROP_NAME
 };
 
+enum {
+	DESTROY,
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static void     class_init          (SeahorseWidgetClass    *klass);
 static void     object_init         (SeahorseWidget         *swidget);
 
+static void     object_dispose     (GObject                *gobject);
 static void     object_finalize     (GObject                *gobject);
 
 static void     object_set_property (GObject                *object,
@@ -67,14 +82,21 @@ G_MODULE_EXPORT gboolean on_widget_delete_event  (GtkWidget             *widget,
                                                        GdkEvent              *event,
                                                        SeahorseWidget        *swidget);
 
-static void     context_destroyed    (GtkObject             *object,
+static void     context_destroyed    (SeahorseContext       *sctx,
                                       SeahorseWidget        *swidget);
 
-static GtkObjectClass *parent_class = NULL;
+static GObjectClass *parent_class = NULL;
 
 /* Hash of widgets with name as key */
 static GHashTable *widgets = NULL;
 
+/**
+ * seahorse_widget_get_type:
+ *
+ * Registers the widget type "SeahorseWidget"
+ *
+ * Returns: The widget identifier
+ */
 GType
 seahorse_widget_get_type (void)
 {
@@ -86,14 +108,20 @@ seahorse_widget_get_type (void)
 			(GClassInitFunc) class_init,
 			NULL, NULL, sizeof (SeahorseWidget), 0, (GInstanceInitFunc) object_init
 		};
-		
-        widget_type = g_type_register_static (GTK_TYPE_OBJECT, "SeahorseWidget", 
-                                              &widget_info, 0);
+
+		widget_type = g_type_register_static (G_TYPE_OBJECT, "SeahorseWidget",
+		                                      &widget_info, 0);
 	}
 	
 	return widget_type;
 }
 
+/**
+* klass: the #SeahorseWidgetClass
+*
+* Initialises the SeahorseWidgetClass
+*
+**/
 static void
 class_init (SeahorseWidgetClass *klass)
 {
@@ -103,29 +131,55 @@ class_init (SeahorseWidgetClass *klass)
 	gobject_class = G_OBJECT_CLASS (klass);
 	
 	gobject_class->constructor = seahorse_widget_constructor;
+	gobject_class->dispose = object_dispose;
 	gobject_class->finalize = object_finalize;
 	gobject_class->set_property = object_set_property;
 	gobject_class->get_property = object_get_property;
-	
-    g_object_class_install_property (gobject_class, PROP_NAME,
-        g_param_spec_string ("name", "Widget name", "Name of gtkbuilder file and main widget",
-                             NULL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (gobject_class, PROP_NAME,
+	           g_param_spec_string ("name", "Widget name", "Name of gtkbuilder file and main widget",
+	                                NULL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	signals[DESTROY] = g_signal_new ("destroy", SEAHORSE_TYPE_WIDGET,
+	                                 G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET (SeahorseWidgetClass, destroy),
+	                                 NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 }
 
-/* Destroy widget when context is destroyed */
+/**
+* sctx: ignored
+* swidget: The swidget being destroyed
+*
+* Destroy widget when context is destroyed
+*
+**/
 static void
-context_destroyed (GtkObject *object, SeahorseWidget *swidget)
+context_destroyed (SeahorseContext *sctx, SeahorseWidget *swidget)
 {
 	seahorse_widget_destroy (swidget);
 }
 
+/**
+* swidget: The #SeahorseWidget being initialised
+*
+* Connects the destroy-signal
+*
+**/
 static void
 object_init (SeahorseWidget *swidget)
 {
-    g_signal_connect_after (SCTX_APP(), "destroy", 
-                G_CALLBACK (context_destroyed), swidget);
+	g_signal_connect_after (SCTX_APP(), "destroy",
+	                        G_CALLBACK (context_destroyed), swidget);
 }
 
+/**
+* type: the #GType to construct
+* n_props: number of properties
+* props: properties
+*
+*
+*
+* Returns the initialised object
+**/
 static GObject*  
 seahorse_widget_constructor (GType type, guint n_props, GObjectConstructParam* props)
 {
@@ -158,8 +212,27 @@ seahorse_widget_constructor (GType type, guint n_props, GObjectConstructParam* p
     return obj;
 }
 
-/* Disconnects callbacks, destroys main window widget,
- * and frees the xml definition and any other data */
+static void
+object_dispose (GObject *object)
+{
+	SeahorseWidget *swidget = SEAHORSE_WIDGET (object);
+
+	if (!swidget->in_destruction) {
+		swidget->in_destruction = TRUE;
+		g_signal_emit (swidget, signals[DESTROY], 0);
+		swidget->in_destruction = FALSE;
+	}
+
+	G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
+/**
+* gobject: The #SeahorseWidget to finalize
+*
+* Disconnects callbacks, destroys main window widget,
+* and frees the xml definition and any other data
+*
+**/
 static void
 object_finalize (GObject *gobject)
 {
@@ -188,6 +261,15 @@ object_finalize (GObject *gobject)
 	G_OBJECT_CLASS (parent_class)->finalize (gobject);
 }
 
+/**
+* object: the object to set the value for
+* prop_id: the id of the property to set
+* value: the value to set
+* pspec: ignored
+*
+* Only property PROP_NAME is available so far
+*
+**/
 static void
 object_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
@@ -212,7 +294,6 @@ object_set_property (GObject *object, guint prop_id, const GValue *value, GParam
         gtk_builder_connect_signals (swidget->gtkbuilder, swidget);
         
         w = GTK_WIDGET (seahorse_widget_get_widget (swidget, swidget->name));
-		GTK_WIDGET_SET_FLAGS (GTK_WINDOW (w), GTK_TOPLEVEL);
         /*TODO: glade_xml_ensure_accel (swidget->gtkbuilder);*/
         
         gtk_window_set_icon_name (GTK_WINDOW (w), "seahorse");
@@ -220,8 +301,18 @@ object_set_property (GObject *object, guint prop_id, const GValue *value, GParam
     }
 }
 
+/**
+* object: the object to get the property for
+* prop_id: ID of the property
+* value: Value to return
+* pspec: ignored
+*
+* Only the property PROP_NAME is available so far
+*
+**/
 static void
 object_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+
 {
 	SeahorseWidget *swidget;
 	swidget = SEAHORSE_WIDGET (object);
@@ -235,12 +326,29 @@ object_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *
 	}
 }
 
+/**
+ * on_widget_help:
+ * @widget: The widget
+ * @swidget: The #SeahorseWidget
+ *
+ * Shows help to the widget
+ *
+ */
 G_MODULE_EXPORT void
 on_widget_help (GtkWidget *widget, SeahorseWidget *swidget)
 {
     seahorse_widget_show_help (swidget);
 }
 
+
+/**
+ * on_widget_closed:
+ * @widget: ignored
+ * @swidget: The #SeahorseWidget
+ *
+ * Closes the widget, calls destroy-function
+ *
+ */
 /* Destroys widget */
 G_MODULE_EXPORT void
 on_widget_closed (GtkWidget *widget, SeahorseWidget *swidget)
@@ -248,6 +356,17 @@ on_widget_closed (GtkWidget *widget, SeahorseWidget *swidget)
 	seahorse_widget_destroy (swidget);
 }
 
+
+/**
+ * on_widget_delete_event:
+ * @widget: the widget being deleted
+ * @event: ignored
+ * @swidget: the #SeahorseWidget
+ *
+ *
+ *
+ * Returns: FALSE
+ */
 /* Closed widget */
 G_MODULE_EXPORT gboolean
 on_widget_delete_event (GtkWidget *widget, GdkEvent *event, SeahorseWidget *swidget)
@@ -261,7 +380,8 @@ on_widget_delete_event (GtkWidget *widget, GdkEvent *event, SeahorseWidget *swid
  * @name: Name of widget, filename part of gtkbuilder file, and name of main window
  * @parent: GtkWindow to make the parent of the new swidget
  *
- * Creates a new #SeahorseWidget.
+ * Creates a new #SeahorseWidget. Date is read from the gtk-builder file
+ * seahorse-%name%.xml
  *
  * Returns: The new #SeahorseWidget, or NULL if the widget already exists
  **/
@@ -289,11 +409,6 @@ seahorse_widget_new (const gchar *name, GtkWindow *parent)
         gtk_window_set_transient_for (window, parent);
     }
 
-    /* We don't care about this floating business */
-    g_object_ref (GTK_OBJECT (swidget));
-    g_object_ref_sink (GTK_OBJECT (swidget));
-    g_object_unref (GTK_OBJECT (swidget));
-
     return swidget;
 }
 
@@ -309,26 +424,29 @@ seahorse_widget_new (const gchar *name, GtkWindow *parent)
 SeahorseWidget*
 seahorse_widget_new_allow_multiple (const gchar *name, GtkWindow *parent)
 {
-    GtkWindow *window;
-    SeahorseWidget *swidget = g_object_new (SEAHORSE_TYPE_WIDGET, "name", name,  NULL);
-    
-    if (parent != NULL) {
-        window = GTK_WINDOW (seahorse_widget_get_widget (swidget, swidget->name));
-        gtk_window_set_transient_for (window, parent);
-    }
+	GtkWindow *window;
+	SeahorseWidget *swidget;
+
+	swidget = g_object_new (SEAHORSE_TYPE_WIDGET, "name", name,  NULL);
+
+	if (parent != NULL) {
+		window = GTK_WINDOW (seahorse_widget_get_widget (swidget, swidget->name));
+		gtk_window_set_transient_for (window, parent);
+	}
 
 	gtk_builder_connect_signals (swidget->gtkbuilder, NULL);
-    
-    /* We don't care about this floating business */
-    g_object_ref (GTK_OBJECT (swidget));
-    g_object_ref_sink (GTK_OBJECT (swidget));
-    g_object_unref (GTK_OBJECT (swidget));
-
-    return swidget;
+	return swidget;
 }
 
+/**
+ * seahorse_widget_find:
+ * @name: The name to look for
+ *
+ * Returns: The widget with the @name or NULL if not found
+ */
 SeahorseWidget*
 seahorse_widget_find (const gchar *name)
+
 {
     /* Check if have widget hash */
     if (widgets != NULL)
@@ -337,7 +455,7 @@ seahorse_widget_find (const gchar *name)
 }
 
 /**
- * seahorse_widget_show_help
+ * seahorse_widget_show_help:
  * @swidget: The #SeahorseWidget.
  * 
  * Show help appropriate for the top level widget.
@@ -370,6 +488,12 @@ seahorse_widget_show_help (SeahorseWidget *swidget)
         g_error_free (error);
 }
 
+/**
+ * seahorse_widget_get_name:
+ * @swidget: The widget to get the name for
+ *
+ * Returns: The name of the widget
+ */
 const gchar*
 seahorse_widget_get_name (SeahorseWidget   *swidget)
 {
@@ -378,7 +502,7 @@ seahorse_widget_get_name (SeahorseWidget   *swidget)
 }
 
 /**
- * seahorse_widget_get_toplevel
+ * seahorse_widget_get_toplevel:
  * @swidget: The seahorse widget
  * 
  * Return the top level widget in this seahorse widget
@@ -425,9 +549,17 @@ seahorse_widget_show (SeahorseWidget *swidget)
     gtk_widget_show (widget);
 }
  
+/**
+ * seahorse_widget_set_visible:
+ * @swidget: the #SeahorseWidget
+ * @identifier: The identifier of the widget to set visibility for
+ * @visible: TRUE to show the widget, FALSE to hide it
+ *
+ */
 void             
 seahorse_widget_set_visible (SeahorseWidget *swidget, const char *identifier,
                              gboolean visible)
+
 {
     GtkWidget *widget = GTK_WIDGET (seahorse_widget_get_widget (swidget, identifier));
     g_return_if_fail (widget != NULL);
@@ -441,7 +573,7 @@ seahorse_widget_set_visible (SeahorseWidget *swidget, const char *identifier,
 /**
  * seahorse_widget_set_sensitive:
  * @swidget: the #SeahorseWidget to find the widget @identifier in
- * @identifier: the name of the widget to et sensitive
+ * @identifier: the name of the widget to set sensitive
  * @sensitive: TRUE if the widget should be switched to sensitive
  *
  * Sets the widget @identifier 's sensitivity to @sensitive
